@@ -1,7 +1,7 @@
 // qsamplerChannelForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -48,7 +48,7 @@ ChannelForm::ChannelForm ( QWidget* pParent )
 	m_ui.setupUi(this);
 
 	// Initialize locals.
-	m_pChannel = NULL;
+	m_pChannel = nullptr;
 
 	m_iDirtySetup = 0;
 	m_iDirtyCount = 0;
@@ -56,7 +56,7 @@ ChannelForm::ChannelForm ( QWidget* pParent )
 //	m_midiDevices.setAutoDelete(true);
 //	m_audioDevices.setAutoDelete(true);
 
-	m_pDeviceForm = NULL;
+	m_pDeviceForm = nullptr;
 
 	const int iRowHeight = m_ui.AudioRoutingTable->fontMetrics().height() + 4;
 	m_ui.AudioRoutingTable->verticalHeader()->setDefaultSectionSize(iRowHeight);
@@ -64,7 +64,7 @@ ChannelForm::ChannelForm ( QWidget* pParent )
 
 	m_ui.AudioRoutingTable->setModel(&m_routingModel);
 	m_ui.AudioRoutingTable->setItemDelegate(&m_routingDelegate);
-#if QT_VERSION >= 0x050000
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 	m_ui.AudioRoutingTable->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);
 #else
 	m_ui.AudioRoutingTable->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
@@ -142,7 +142,7 @@ ChannelForm::~ChannelForm()
 {
 	if (m_pDeviceForm)
 		delete m_pDeviceForm;
-	m_pDeviceForm = NULL;
+	m_pDeviceForm = nullptr;
 
 	qDeleteAll(m_midiDevices);
 	m_midiDevices.clear();
@@ -160,22 +160,22 @@ void ChannelForm::setup ( Channel *pChannel )
 	m_iDirtySetup = 0;
 	m_iDirtyCount = 0;
 
-	if (m_pChannel == NULL)
+	if (m_pChannel == nullptr)
 		return;
 
 	// It can be a brand new channel, remember?
 	const bool bNew = (m_pChannel->channelID() < 0);
-	setWindowTitle(QSAMPLER_TITLE ": " + m_pChannel->channelName());
+	setWindowTitle(m_pChannel->channelName());
 
 	// Check if we're up and connected.
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	Options *pOptions = pMainForm->options();
-	if (pOptions == NULL)
+	if (pOptions == nullptr)
 		return;
 
 	// Avoid nested changes.
@@ -267,9 +267,8 @@ void ChannelForm::setup ( Channel *pChannel )
 	}
 	selectMidiDriverItem(sMidiDriver);
 	if (!bNew) {
-		m_ui.MidiDeviceComboBox->setItemText(
-			m_ui.MidiDeviceComboBox->currentIndex(),
-			midiDevice.deviceName());
+		m_ui.MidiDeviceComboBox->setCurrentIndex(
+			m_ui.MidiDeviceComboBox->findData(midiDevice.deviceID()));
 	}
 	selectMidiDeviceItem(m_ui.MidiDeviceComboBox->currentIndex());
 	// MIDI input port...
@@ -314,9 +313,8 @@ void ChannelForm::setup ( Channel *pChannel )
 	}
 	selectAudioDriverItem(sAudioDriver);
 	if (!bNew) {
-		m_ui.AudioDeviceComboBox->setItemText(
-			m_ui.AudioDeviceComboBox->currentIndex(),
-			audioDevice.deviceName());
+		m_ui.AudioDeviceComboBox->setCurrentIndex(
+			m_ui.AudioDeviceComboBox->findData(audioDevice.deviceID()));
 	}
 	selectAudioDeviceItem(m_ui.AudioDeviceComboBox->currentIndex());
 
@@ -350,17 +348,17 @@ void ChannelForm::setup ( Channel *pChannel )
 // Accept settings (OK button slot).
 void ChannelForm::accept (void)
 {
-	if (m_pChannel == NULL)
+	if (m_pChannel == nullptr)
 		return;
 
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	Options *pOptions = pMainForm->options();
-	if (pOptions == NULL)
+	if (pOptions == nullptr)
 		return;
 
 	// Flush any pending editing...
@@ -377,12 +375,15 @@ void ChannelForm::accept (void)
 			if (!m_pChannel->setAudioDriver(m_ui.AudioDriverComboBox->currentText()))
 				iErrors++;
 		} else {
-			Device *pDevice = NULL;
+			Device *pDevice = nullptr;
 			const int iAudioItem = m_ui.AudioDeviceComboBox->currentIndex();
-			if (iAudioItem >= 0 && iAudioItem < m_audioDevices.count())
-				pDevice = m_audioDevices.at(iAudioItem);
+			if (iAudioItem >= 0) {
+				const int iAudioDevice
+					= m_ui.AudioDeviceComboBox->itemData(iAudioItem).toInt();
+				pDevice = m_audioDevices.value(iAudioDevice, nullptr);
+			}
 			ChannelRoutingMap routingMap = m_routingModel.routingMap();
-			if (pDevice == NULL)
+			if (pDevice == nullptr)
 				iErrors++;
 			else if (!m_pChannel->setAudioDevice(pDevice->deviceID()))
 				iErrors++;
@@ -401,11 +402,14 @@ void ChannelForm::accept (void)
 			if (!m_pChannel->setMidiDriver(m_ui.MidiDriverComboBox->currentText()))
 				iErrors++;
 		} else {
-			Device *pDevice = NULL;
+			Device *pDevice = nullptr;
 			const int iMidiItem = m_ui.MidiDeviceComboBox->currentIndex();
-			if (iMidiItem >= 0 && iMidiItem < m_midiDevices.count())
-				pDevice = m_midiDevices.at(iMidiItem);
-			if (pDevice == NULL)
+			if (iMidiItem >= 0) {
+				const int iMidiDevice
+					= m_ui.MidiDeviceComboBox->itemData(iMidiItem).toInt();
+				pDevice = m_midiDevices.value(iMidiDevice, nullptr);
+			}
+			if (pDevice == nullptr)
 				iErrors++;
 			else if (!m_pChannel->setMidiDevice(pDevice->deviceID()))
 				iErrors++;
@@ -457,7 +461,7 @@ void ChannelForm::reject (void)
 	// Check if there's any pending changes...
 	if (m_iDirtyCount > 0) {
 		switch (QMessageBox::warning(this,
-			QSAMPLER_TITLE ": " + tr("Warning"),
+			tr("Warning"),
 			tr("Some channel settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
 			QMessageBox::Apply |
@@ -483,13 +487,13 @@ void ChannelForm::reject (void)
 void ChannelForm::openInstrumentFile (void)
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	Options *pOptions = pMainForm->options();
-	if (pOptions == NULL)
+	if (pOptions == nullptr)
 		return;
 
 	// FIXME: the instrument file filters should be restricted,
@@ -506,7 +510,7 @@ void ChannelForm::openInstrumentFile (void)
 	const QString& filter = filters.join(";;");
 
 	QString sInstrumentFile = QFileDialog::getOpenFileName(this,
-		QSAMPLER_TITLE ": " + tr("Instrument files"), // Caption.
+		tr("Instrument files"),   // Caption.
 		pOptions->sInstrumentDir, // Start here.
 		filter                    // File filter.
 	);
@@ -523,13 +527,13 @@ void ChannelForm::openInstrumentFile (void)
 void ChannelForm::updateInstrumentName (void)
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	Options *pOptions = pMainForm->options();
-	if (pOptions == NULL)
+	if (pOptions == nullptr)
 		return;
 
 	// Finally this better idea would be to use libgig
@@ -551,13 +555,13 @@ void ChannelForm::setupDevice ( Device *pDevice,
 	const QString& sDriverName )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	// Create the device form if not already...
-	if (m_pDeviceForm == NULL) {
+	if (m_pDeviceForm == nullptr) {
 		m_pDeviceForm = new DeviceForm(this, Qt::Dialog);
 		m_pDeviceForm->setAttribute(Qt::WA_ShowModal);
 		QObject::connect(m_pDeviceForm, SIGNAL(devicesChanged()),
@@ -579,21 +583,18 @@ void ChannelForm::setupDevice ( Device *pDevice,
 void ChannelForm::selectMidiDriverItem ( const QString& sMidiDriver )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	const QString sDriverName = sMidiDriver.toUpper();
 
 	// Save current device id.
 	int iDeviceID = 0;
-	Device *pDevice = NULL;
 	int iMidiItem = m_ui.MidiDeviceComboBox->currentIndex();
-	if (iMidiItem >= 0 && iMidiItem < m_midiDevices.count())
-		pDevice = m_midiDevices.at(iMidiItem);
-	if (pDevice)
-		iDeviceID = pDevice->deviceID();
+	if (iMidiItem >= 0)
+		iDeviceID = m_ui.MidiDeviceComboBox->itemData(iMidiItem).toInt();
 
 	// Clean maplist.
 	m_ui.MidiDeviceComboBox->clear();
@@ -601,14 +602,19 @@ void ChannelForm::selectMidiDriverItem ( const QString& sMidiDriver )
 	m_midiDevices.clear();
 
 	// Populate with the current ones...
+	Device *pDevice = nullptr;
 	const QPixmap midiPixmap(":/images/midi2.png");
 	int *piDeviceIDs = Device::getDevices(pMainForm->client(), Device::Midi);
-	for (int i = 0; piDeviceIDs && piDeviceIDs[i] >= 0; i++) {
+	for (int i = 0; piDeviceIDs && piDeviceIDs[i] >= 0; ++i) {
 		pDevice = new Device(Device::Midi, piDeviceIDs[i]);
 		if (pDevice->driverName().toUpper() == sDriverName) {
-			m_ui.MidiDeviceComboBox->insertItem(0,
+			const int iMidiDevice = pDevice->deviceID();
+			m_ui.MidiDeviceComboBox->addItem(
 				midiPixmap, pDevice->deviceName());
-			m_midiDevices.append(pDevice);
+			m_ui.MidiDeviceComboBox->setItemData(i, iMidiDevice);
+			m_midiDevices.insert(iMidiDevice, pDevice);
+			if (iMidiDevice == iDeviceID)
+				iMidiItem = i;
 		} else {
 			delete pDevice;
 		}
@@ -618,23 +624,15 @@ void ChannelForm::selectMidiDriverItem ( const QString& sMidiDriver )
 	const bool bEnabled = !m_midiDevices.isEmpty();
 	if (bEnabled) {
 		// Select the previous current device...
-		iMidiItem = 0;
-		QListIterator<Device *> iter(m_midiDevices);
-		while (iter.hasNext()) {
-			pDevice = iter.next();
-			if (pDevice->deviceID() == iDeviceID) {
-				m_ui.MidiDeviceComboBox->setCurrentIndex(iMidiItem);
-				selectMidiDeviceItem(iMidiItem);
-				break;
-			}
-			iMidiItem++;
-		}
+		m_ui.MidiDeviceComboBox->setCurrentIndex(iMidiItem);
+		selectMidiDeviceItem(iMidiItem);
 	} else {
 		m_ui.MidiDeviceComboBox->insertItem(0,
 			tr("(New MIDI %1 device)").arg(sMidiDriver));
+		m_ui.MidiDeviceComboBox->setItemData(0, -1);
 	}
-	m_ui.MidiDeviceTextLabel->setEnabled(bEnabled);
-	m_ui.MidiDeviceComboBox->setEnabled(bEnabled);
+//	m_ui.MidiDeviceTextLabel->setEnabled(bEnabled);
+//	m_ui.MidiDeviceComboBox->setEnabled(bEnabled);
 }
 
 
@@ -652,9 +650,12 @@ void ChannelForm::selectMidiDriver ( const QString& sMidiDriver )
 // Select MIDI device item.
 void ChannelForm::selectMidiDeviceItem ( int iMidiItem )
 {
-	Device *pDevice = NULL;
-	if (iMidiItem >= 0 && iMidiItem < m_midiDevices.count())
-		pDevice = m_midiDevices.at(iMidiItem);
+	Device *pDevice = nullptr;
+	if (iMidiItem >= 0) {
+		const int iMidiDevice
+			= m_ui.MidiDeviceComboBox->itemData(iMidiItem).toInt();
+		pDevice = m_midiDevices.value(iMidiDevice, nullptr);
+	}
 	if (pDevice) {
 		const DeviceParamMap& params = pDevice->params();
 		const int iPorts = params["PORTS"].value.toInt();
@@ -680,10 +681,13 @@ void ChannelForm::selectMidiDevice ( int iMidiItem )
 // MIDI device options.
 void ChannelForm::setupMidiDevice (void)
 {
-	Device *pDevice = NULL;
+	Device *pDevice = nullptr;
 	const int iMidiItem = m_ui.MidiDeviceComboBox->currentIndex();
-	if (iMidiItem >= 0 && iMidiItem < m_midiDevices.count())
-		pDevice = m_midiDevices.at(iMidiItem);
+	if (iMidiItem >= 0) {
+		const int iMidiDevice
+			= m_ui.MidiDeviceComboBox->itemData(iMidiItem).toInt();
+		pDevice = m_midiDevices.value(iMidiDevice, nullptr);
+	}
 	setupDevice(pDevice,
 		Device::Midi, m_ui.MidiDriverComboBox->currentText());
 }
@@ -693,21 +697,18 @@ void ChannelForm::setupMidiDevice (void)
 void ChannelForm::selectAudioDriverItem ( const QString& sAudioDriver )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	const QString sDriverName = sAudioDriver.toUpper();
 
 	// Save current device id.
 	int iDeviceID = 0;
-	Device *pDevice = NULL;
 	int iAudioItem = m_ui.AudioDeviceComboBox->currentIndex();
-	if (iAudioItem >= 0 && iAudioItem < m_audioDevices.count())
-		pDevice = m_audioDevices.at(iAudioItem);
-	if (pDevice)
-		iDeviceID = pDevice->deviceID();
+	if (iAudioItem >= 0)
+		iDeviceID = m_ui.AudioDeviceComboBox->itemData(iAudioItem).toInt();
 
 	// Clean maplist.
 	m_ui.AudioDeviceComboBox->clear();
@@ -715,15 +716,19 @@ void ChannelForm::selectAudioDriverItem ( const QString& sAudioDriver )
 	m_audioDevices.clear();
 
 	// Populate with the current ones...
+	Device *pDevice = nullptr;
 	const QPixmap audioPixmap(":/images/audio2.png");
-	int *piDeviceIDs = Device::getDevices(pMainForm->client(),
-		Device::Audio);
-	for (int i = 0; piDeviceIDs && piDeviceIDs[i] >= 0; i++) {
+	int *piDeviceIDs = Device::getDevices(pMainForm->client(), Device::Audio);
+	for (int i = 0; piDeviceIDs && piDeviceIDs[i] >= 0; ++i) {
 		pDevice = new Device(Device::Audio, piDeviceIDs[i]);
 		if (pDevice->driverName().toUpper() == sDriverName) {
-			m_ui.AudioDeviceComboBox->insertItem(0,
+			const int iAudioDevice = pDevice->deviceID();
+			m_ui.AudioDeviceComboBox->addItem(
 				audioPixmap, pDevice->deviceName());
-			m_audioDevices.append(pDevice);
+			m_ui.AudioDeviceComboBox->setItemData(i, iAudioDevice);
+			m_audioDevices.insert(iAudioDevice, pDevice);
+			if (iAudioDevice == iDeviceID)
+				iAudioItem = i;
 		} else {
 			delete pDevice;
 		}
@@ -733,24 +738,15 @@ void ChannelForm::selectAudioDriverItem ( const QString& sAudioDriver )
 	const bool bEnabled = !m_audioDevices.isEmpty();
 	if (bEnabled) {
 		// Select the previous current device...
-		iAudioItem = 0;
-		QListIterator<Device *> iter(m_audioDevices);
-		while (iter.hasNext()) {
-			pDevice = iter.next();
-			if (pDevice->deviceID() == iDeviceID) {
-				m_ui.AudioDeviceComboBox->setCurrentIndex(iAudioItem);
-				selectAudioDeviceItem(iAudioItem);
-				break;
-			}
-			iAudioItem++;
-		}
+		m_ui.AudioDeviceComboBox->setCurrentIndex(iAudioItem);
+		selectAudioDeviceItem(iAudioItem);
 	} else {
 		m_ui.AudioDeviceComboBox->insertItem(0,
 			tr("(New Audio %1 device)").arg(sAudioDriver));
-		//m_ui.AudioRoutingTable->setNumRows(0);
+		m_ui.AudioDeviceComboBox->setItemData(0, -1);
 	}
-	m_ui.AudioDeviceTextLabel->setEnabled(bEnabled);
-	m_ui.AudioDeviceComboBox->setEnabled(bEnabled);
+//	m_ui.AudioDeviceTextLabel->setEnabled(bEnabled);
+//	m_ui.AudioDeviceComboBox->setEnabled(bEnabled);
 	m_ui.AudioRoutingTable->setEnabled(bEnabled);
 }
 
@@ -769,9 +765,12 @@ void ChannelForm::selectAudioDriver ( const QString& sAudioDriver )
 // Select Audio device item.
 void ChannelForm::selectAudioDeviceItem ( int iAudioItem )
 {
-	Device *pDevice = NULL;
-	if (iAudioItem >= 0 && iAudioItem < m_audioDevices.count())
-		pDevice = m_audioDevices.at(iAudioItem);
+	Device *pDevice = nullptr;
+	if (iAudioItem >= 0) {
+		const int iAudioDevice
+			= m_ui.AudioDeviceComboBox->itemData(iAudioItem).toInt();
+		pDevice = m_audioDevices.value(iAudioDevice, nullptr);
+	}
 	if (pDevice) {
 		// Refresh the audio routing table.
 		m_routingModel.refresh(pDevice, m_pChannel->audioRouting());
@@ -795,10 +794,13 @@ void ChannelForm::selectAudioDevice ( int iAudioItem )
 // Audio device options.
 void ChannelForm::setupAudioDevice (void)
 {
-	Device *pDevice = NULL;
+	Device *pDevice = nullptr;
 	const int iAudioItem = m_ui.AudioDeviceComboBox->currentIndex();
-	if (iAudioItem >= 0 && iAudioItem < m_audioDevices.count())
-		pDevice = m_audioDevices.at(iAudioItem);
+	if (iAudioItem >= 0) {
+		const int iAudioDevice
+			= m_ui.AudioDeviceComboBox->itemData(iAudioItem).toInt();
+		pDevice = m_audioDevices.value(iAudioDevice, nullptr);
+	}
 	setupDevice(pDevice,
 		Device::Audio, m_ui.AudioDriverComboBox->currentText());
 }
