@@ -1,7 +1,7 @@
 // qsamplerChannelStrip.h
 //
 /****************************************************************************
-   Copyright (C) 2004-201, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008, 2014 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -44,7 +44,7 @@ class ChannelStrip : public QWidget
 
 public:
 
-	ChannelStrip(QWidget* pParent = NULL, Qt::WindowFlags wflags = 0);
+	ChannelStrip(QWidget* pParent = nullptr, Qt::WindowFlags wflags = 0);
 	~ChannelStrip();
 
 	void setup(Channel *pChannel);
