@@ -1,7 +1,7 @@
 // qsamplerDevice.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -45,17 +45,11 @@ DeviceParam::DeviceParam ( lscp_param_info_t *pParamInfo,
 }
 
 
-// Default destructor.
-DeviceParam::~DeviceParam (void)
-{
-}
-
-
 // Initializer.
 void DeviceParam::setParam ( lscp_param_info_t *pParamInfo,
 	const char *pszValue )
 {
-	if (pParamInfo == NULL)
+	if (pParamInfo == nullptr)
 		return;
 
 	// Info structure field members.
@@ -65,11 +59,11 @@ void DeviceParam::setParam ( lscp_param_info_t *pParamInfo,
 	if (pParamInfo->description)
 		description = pParamInfo->description;
 	else
-		description = QString::null;
+		description.clear();
 
-	mandatory = (bool) pParamInfo->mandatory;
-	fix = (bool) pParamInfo->fix;
-	multiplicity = (bool) pParamInfo->multiplicity;
+	mandatory = bool(pParamInfo->mandatory);
+	fix = bool(pParamInfo->fix);
+	multiplicity = bool(pParamInfo->multiplicity);
 
 	depends.clear();
 	for (int i = 0; pParamInfo->depends && pParamInfo->depends[i]; i++)
@@ -78,17 +72,17 @@ void DeviceParam::setParam ( lscp_param_info_t *pParamInfo,
 	if (pParamInfo->defaultv)
 		defaultv = pParamInfo->defaultv;
 	else
-		defaultv = QString::null;
+		defaultv.clear();
 
 	if (pParamInfo->range_min)
 		range_min = pParamInfo->range_min;
 	else
-		range_min = QString::null;
+		range_min.clear();
 
 	if (pParamInfo->range_max)
 		range_max = pParamInfo->range_max;
 	else
-		range_max = QString::null;
+		range_max.clear();
 
 	possibilities.clear();
 	for (int i = 0; pParamInfo->possibilities && pParamInfo->possibilities[i]; i++)
@@ -98,7 +92,7 @@ void DeviceParam::setParam ( lscp_param_info_t *pParamInfo,
 	if (pszValue)
 		value = pszValue;
 	else
-		value = QString::null;
+		value.clear();
 }
 
 
@@ -137,9 +131,9 @@ Device::Device ( const Device& device )
 void Device::setDevice ( DeviceType deviceType, int iDeviceID )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	// Device id and type should be always set.
@@ -152,27 +146,27 @@ void Device::setDevice ( DeviceType deviceType, int iDeviceID )
 	m_ports.clear();
 
 	// Retrieve device info, if any.
-	lscp_device_info_t *pDeviceInfo = NULL;
+	lscp_device_info_t *pDeviceInfo = nullptr;
 	switch (deviceType) {
 	case Device::Audio:
 		m_sDeviceType = QObject::tr("Audio");
 		if (m_iDeviceID >= 0 && (pDeviceInfo = ::lscp_get_audio_device_info(
-				pMainForm->client(), m_iDeviceID)) == NULL)
+				pMainForm->client(), m_iDeviceID)) == nullptr)
 			appendMessagesClient("lscp_get_audio_device_info");
 		break;
 	case Device::Midi:
 		m_sDeviceType = QObject::tr("MIDI");
 		if (m_iDeviceID >= 0 && (pDeviceInfo = ::lscp_get_midi_device_info(
-				pMainForm->client(), m_iDeviceID)) == NULL)
+				pMainForm->client(), m_iDeviceID)) == nullptr)
 			appendMessagesClient("lscp_get_midi_device_info");
 		break;
 	case Device::None:
-		m_sDeviceType = QString::null;
+		m_sDeviceType.clear();
 		break;
 	}
 	// If we're bogus, bail out...
-	if (pDeviceInfo == NULL) {
-		m_sDriverName = QString::null;
+	if (pDeviceInfo == nullptr) {
+		m_sDriverName.clear();
 		m_sDeviceName = QObject::tr("New %1 device").arg(m_sDeviceType);
 		return;
 	}
@@ -185,18 +179,18 @@ void Device::setDevice ( DeviceType deviceType, int iDeviceID )
 	// Grab device parameters...
 	for (int i = 0; pDeviceInfo->params && pDeviceInfo->params[i].key; i++) {
 		const QString sParam = pDeviceInfo->params[i].key;
-		lscp_param_info_t *pParamInfo = NULL;
+		lscp_param_info_t *pParamInfo = nullptr;
 		switch (deviceType) {
 		case Device::Audio:
 			if ((pParamInfo = ::lscp_get_audio_driver_param_info(
 					pMainForm->client(), m_sDriverName.toUtf8().constData(),
-					sParam.toUtf8().constData(), NULL)) == NULL)
+					sParam.toUtf8().constData(), nullptr)) == nullptr)
 				appendMessagesClient("lscp_get_audio_driver_param_info");
 			break;
 		case Device::Midi:
 			if ((pParamInfo = ::lscp_get_midi_driver_param_info(
 					pMainForm->client(), m_sDriverName.toUtf8().constData(),
-					sParam.toUtf8().constData(), NULL)) == NULL)
+					sParam.toUtf8().constData(), nullptr)) == nullptr)
 				appendMessagesClient("lscp_get_midi_driver_param_info");
 			break;
 		case Device::None:
@@ -219,9 +213,9 @@ void Device::setDevice ( DeviceType deviceType, int iDeviceID )
 void Device::setDriver ( const QString& sDriverName )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	// Valid only for scratch devices.
@@ -234,16 +228,16 @@ void Device::setDriver ( const QString& sDriverName )
 	m_ports.clear();
 
 	// Retrieve driver info, if any.
-	lscp_driver_info_t *pDriverInfo = NULL;
+	lscp_driver_info_t *pDriverInfo = nullptr;
 	switch (m_deviceType) {
 	case Device::Audio:
 		if ((pDriverInfo = ::lscp_get_audio_driver_info(pMainForm->client(),
-				sDriverName.toUtf8().constData())) == NULL)
+				sDriverName.toUtf8().constData())) == nullptr)
 			appendMessagesClient("lscp_get_audio_driver_info");
 		break;
 	case Device::Midi:
 		if ((pDriverInfo = ::lscp_get_midi_driver_info(pMainForm->client(),
-				sDriverName.toUtf8().constData())) == NULL)
+				sDriverName.toUtf8().constData())) == nullptr)
 			appendMessagesClient("lscp_get_midi_driver_info");
 		break;
 	case Device::None:
@@ -251,7 +245,7 @@ void Device::setDriver ( const QString& sDriverName )
 	}
 
 	// If we're bogus, bail out...
-	if (pDriverInfo == NULL)
+	if (pDriverInfo == nullptr)
 		return;
 
 	// Remember device parameters...
@@ -260,18 +254,18 @@ void Device::setDriver ( const QString& sDriverName )
 	// Grab driver parameters...
 	for (int i = 0; pDriverInfo->parameters && pDriverInfo->parameters[i]; i++) {
 		const QString sParam = pDriverInfo->parameters[i];
-		lscp_param_info_t *pParamInfo = NULL;
+		lscp_param_info_t *pParamInfo = nullptr;
 		switch (m_deviceType) {
 		case Device::Audio:
 			if ((pParamInfo = ::lscp_get_audio_driver_param_info(
 					pMainForm->client(), sDriverName.toUtf8().constData(),
-					sParam.toUtf8().constData(), NULL)) == NULL)
+					sParam.toUtf8().constData(), nullptr)) == nullptr)
 				appendMessagesClient("lscp_get_audio_driver_param_info");
 			break;
 		case Device::Midi:
 			if ((pParamInfo = ::lscp_get_midi_driver_param_info(
 					pMainForm->client(), sDriverName.toUtf8().constData(),
-					sParam.toUtf8().constData(), NULL)) == NULL)
+					sParam.toUtf8().constData(), nullptr)) == nullptr)
 				appendMessagesClient("lscp_get_midi_driver_param_info");
 			break;
 		case Device::None:
@@ -326,9 +320,9 @@ bool Device::setParam ( const QString& sParam,
 	const QString& sValue )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return false;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return false;
 
 	// Set proper device parameter.
@@ -336,7 +330,7 @@ bool Device::setParam ( const QString& sParam,
 
 	// If the device already exists, things get immediate...
 	int iRefresh = 0;
-	if (m_iDeviceID >= 0 && sValue != QString::null) {
+	if (m_iDeviceID >= 0 && !sValue.isEmpty()) {
 
 		// we need temporary byte arrrays with the final strings, because
 		// i.e. QString::toUtf8() only returns a temporary object and the
@@ -403,9 +397,9 @@ DevicePortList& Device::ports (void)
 bool Device::createDevice (void)
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return false;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return false;
 
 	// we need temporary lists with the final strings, because i.e.
@@ -416,7 +410,7 @@ bool Device::createDevice (void)
 
 	DeviceParamMap::ConstIterator iter;
 	for (iter = m_params.begin(); iter != m_params.end(); ++iter) {
-		if (iter.value().value == QString::null) continue;
+		if (iter.value().value.isEmpty()) continue;
 		finalKeys.push_back(iter.key().toUtf8());
 		finalVals.push_back(iter.value().value.toUtf8());
 	}
@@ -431,8 +425,8 @@ bool Device::createDevice (void)
 		pParams[iParam].value = (char *) finalVals[iParam].constData();
 	}
 	// Null terminated.
-	pParams[iParam].key   = NULL;
-	pParams[iParam].value = NULL;
+	pParams[iParam].key   = nullptr;
+	pParams[iParam].value = nullptr;
 
 	// Now it depends on the device type...
 	switch (m_deviceType) {
@@ -451,7 +445,7 @@ bool Device::createDevice (void)
 	}
 
 	// Free used parameter array.
-	delete pParams;
+	delete[] pParams;
 
 	// Show result.
 	if (m_iDeviceID >= 0) {
@@ -471,9 +465,9 @@ bool Device::createDevice (void)
 bool Device::deleteDevice (void)
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return false;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return false;
 
 	// Now it depends on the device type...
@@ -573,9 +567,9 @@ int Device::refreshDepends ( const QString& sParam )
 int Device::refreshParam ( const QString& sParam )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return 0;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return 0;
 
 	// Check if we have dependencies...
@@ -606,8 +600,8 @@ int Device::refreshParam ( const QString& sParam )
 		++iDepend;
 	}
 	// Null terminated.
-	pDepends[iDepend].key   = NULL;
-	pDepends[iDepend].value = NULL;
+	pDepends[iDepend].key   = nullptr;
+	pDepends[iDepend].value = nullptr;
 
 	// FIXME: Some parameter dependencies (e.g.ALSA CARD)
 	// are blocking for no reason, causing potential timeout-crashes.
@@ -615,18 +609,18 @@ int Device::refreshParam ( const QString& sParam )
 	// carried out for scratch devices...
 
 	// Retrieve some modern parameters...
-	lscp_param_info_t *pParamInfo = NULL;
+	lscp_param_info_t *pParamInfo = nullptr;
 	switch (m_deviceType) {
 	case Device::Audio:
 		if ((pParamInfo = ::lscp_get_audio_driver_param_info(
 				pMainForm->client(), m_sDriverName.toUtf8().constData(),
-				sParam.toUtf8().constData(), pDepends)) == NULL)
+				sParam.toUtf8().constData(), pDepends)) == nullptr)
 			appendMessagesClient("lscp_get_audio_driver_param_info");
 		break;
 	case Device::Midi:
 		if ((pParamInfo = ::lscp_get_midi_driver_param_info(
 				pMainForm->client(), m_sDriverName.toUtf8().constData(),
-				sParam.toUtf8().constData(), pDepends)) == NULL)
+				sParam.toUtf8().constData(), pDepends)) == nullptr)
 			appendMessagesClient("lscp_get_midi_driver_param_info");
 		break;
 	case Device::None:
@@ -634,12 +628,12 @@ int Device::refreshParam ( const QString& sParam )
 	}
 	if (pParamInfo) {
 		param = DeviceParam(pParamInfo,
-			param.value.isEmpty() ? NULL : param.value.toUtf8().constData());
+			param.value.isEmpty() ? nullptr : param.value.toUtf8().constData());
 		iRefresh++;
 	}
 
 	// Free used parameter array.
-	delete pDepends;
+	delete[] pDepends;
 
 	// Return whether the parameters has been changed...
 	return iRefresh;
@@ -688,7 +682,7 @@ void Device::appendMessagesClient( const QString& s ) const
 int *Device::getDevices ( lscp_client_t *pClient,
 	DeviceType deviceType )
 {
-	int *piDeviceIDs = NULL;
+	int *piDeviceIDs = nullptr;
 	switch (deviceType) {
 	case Device::Audio:
 		piDeviceIDs = ::lscp_list_audio_devices(pClient);
@@ -720,7 +714,7 @@ QStringList Device::getDrivers ( lscp_client_t *pClient,
 {
 	QStringList drivers;
 
-	const char **ppszDrivers = NULL;
+	const char **ppszDrivers = nullptr;
 	switch (deviceType) {
 	case Device::Audio:
 		ppszDrivers = ::lscp_list_available_audio_drivers(pClient);
@@ -760,9 +754,9 @@ DevicePort::~DevicePort (void)
 void DevicePort::setDevicePort ( int iPortID )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return;
 
 	// Device port id should be always set.
@@ -772,16 +766,16 @@ void DevicePort::setDevicePort ( int iPortID )
 	m_params.clear();
 
 	// Retrieve device port/channel info, if any.
-	lscp_device_port_info_t *pPortInfo = NULL;
+	lscp_device_port_info_t *pPortInfo = nullptr;
 	switch (m_device.deviceType()) {
 	case Device::Audio:
 		if ((pPortInfo = ::lscp_get_audio_channel_info(pMainForm->client(),
-				m_device.deviceID(), m_iPortID)) == NULL)
+				m_device.deviceID(), m_iPortID)) == nullptr)
 			m_device.appendMessagesClient("lscp_get_audio_channel_info");
 		break;
 	case Device::Midi:
 		if ((pPortInfo = ::lscp_get_midi_port_info(pMainForm->client(),
-				m_device.deviceID(), m_iPortID)) == NULL)
+				m_device.deviceID(), m_iPortID)) == nullptr)
 			m_device.appendMessagesClient("lscp_get_midi_port_info");
 		break;
 	case Device::None:
@@ -789,8 +783,8 @@ void DevicePort::setDevicePort ( int iPortID )
 	}
 
 	// If we're bogus, bail out...
-	if (pPortInfo == NULL) {
-		m_sPortName = QString::null;
+	if (pPortInfo == nullptr) {
+		m_sPortName.clear();
 		return;
 	}
 
@@ -801,18 +795,18 @@ void DevicePort::setDevicePort ( int iPortID )
 	m_params.clear();
 	for (int i = 0; pPortInfo->params && pPortInfo->params[i].key; i++) {
 		const QString sParam = pPortInfo->params[i].key;
-		lscp_param_info_t *pParamInfo = NULL;
+		lscp_param_info_t *pParamInfo = nullptr;
 		switch (m_device.deviceType()) {
 		case Device::Audio:
 			if ((pParamInfo = ::lscp_get_audio_channel_param_info(
 					pMainForm->client(), m_device.deviceID(),
-					m_iPortID, sParam.toUtf8().constData())) == NULL)
+					m_iPortID, sParam.toUtf8().constData())) == nullptr)
 				m_device.appendMessagesClient("lscp_get_audio_channel_param_info");
 			break;
 		case Device::Midi:
 			if ((pParamInfo = ::lscp_get_midi_port_param_info(
 					pMainForm->client(), m_device.deviceID(),
-					m_iPortID, sParam.toUtf8().constData())) == NULL)
+					m_iPortID, sParam.toUtf8().constData())) == nullptr)
 				m_device.appendMessagesClient("lscp_get_midi_port_param_info");
 			break;
 		case Device::None:
@@ -849,9 +843,9 @@ bool DevicePort::setParam ( const QString& sParam,
 	const QString& sValue )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return false;
-	if (pMainForm->client() == NULL)
+	if (pMainForm->client() == nullptr)
 		return false;
 
 	// Set proper port/channel parameter.
@@ -970,7 +964,7 @@ Device& DeviceItem::device ()
 AbstractDeviceParamModel::AbstractDeviceParamModel ( QObject* pParent )
 	: QAbstractTableModel(pParent), m_bEditable(false)
 {
-	m_pParams = NULL;
+	m_pParams = nullptr;
 }
 
 
@@ -1018,7 +1012,7 @@ void AbstractDeviceParamModel::refresh (
 	m_pParams   = pParams;
 	m_bEditable = bEditable;
 	// inform the outer world (QTableView) that our data changed
-#if QT_VERSION < 0x050000
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	QAbstractTableModel::reset();
 #else
 	QAbstractTableModel::beginResetModel();
@@ -1029,9 +1023,9 @@ void AbstractDeviceParamModel::refresh (
 
 void AbstractDeviceParamModel::clear (void)
 {
-	m_pParams = NULL;
+	m_pParams = nullptr;
 	// inform the outer world (QTableView) that our data changed
-#if QT_VERSION < 0x050000
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	QAbstractTableModel::reset();
 #else
 	QAbstractTableModel::beginResetModel();
@@ -1047,7 +1041,7 @@ void AbstractDeviceParamModel::clear (void)
 DeviceParamModel::DeviceParamModel ( QObject *pParent )
 	: AbstractDeviceParamModel(pParent)
 {
-	m_pDevice = NULL;
+	m_pDevice = nullptr;
 }
 
 QVariant DeviceParamModel::data (
@@ -1092,7 +1086,7 @@ void DeviceParamModel::refresh ( Device* pDevice, bool bEditable )
 void DeviceParamModel::clear (void)
 {
 	AbstractDeviceParamModel::clear();
-	m_pDevice = NULL;
+	m_pDevice = nullptr;
 }
 
 
@@ -1103,7 +1097,7 @@ void DeviceParamModel::clear (void)
 PortParamModel::PortParamModel ( QObject *pParent)
 	: AbstractDeviceParamModel(pParent)
 {
-	m_pPort = NULL;
+	m_pPort = nullptr;
 }
 
 QVariant PortParamModel::data ( const QModelIndex &index, int role ) const
@@ -1147,7 +1141,7 @@ void PortParamModel::refresh ( DevicePort* pPort, bool bEditable )
 void PortParamModel::clear (void)
 {
 	AbstractDeviceParamModel::clear();
-	m_pPort = NULL;
+	m_pPort = nullptr;
 }
 
 
@@ -1165,12 +1159,13 @@ QWidget* DeviceParamDelegate::createEditor ( QWidget *pParent,
 	const QStyleOptionViewItem& /* option */, const QModelIndex& index ) const
 {
 	if (!index.isValid())
-		return NULL;
+		return nullptr;
 
 	DeviceParameterRow r = index.model()->data(index,
 		Qt::DisplayRole).value<DeviceParameterRow>();
 
 	const bool bEnabled = (r.alive) ? !r.param.fix : true;
+	const bool bFix = r.param.fix;
 
 	QString val = (r.alive) ? r.param.value : r.param.defaultv;
 
@@ -1180,9 +1175,10 @@ QWidget* DeviceParamDelegate::createEditor ( QWidget *pParent,
 		case 1: {
 			if (r.param.type == LSCP_TYPE_BOOL) {
 				QCheckBox *pCheckBox = new QCheckBox(pParent);
-				if (val != QString::null)
+				if (!val.isEmpty())
 					pCheckBox->setChecked(val.toLower() == "true");
 				pCheckBox->setEnabled(bEnabled);
+				pCheckBox->setCheckable(!bFix);
 				return pCheckBox;
 			} else if (r.param.possibilities.count() > 0) {
 				QStringList opts = r.param.possibilities;
@@ -1195,6 +1191,7 @@ QWidget* DeviceParamDelegate::createEditor ( QWidget *pParent,
 				else
 					pComboBox->setCurrentIndex(pComboBox->findText(val));
 				pComboBox->setEnabled(bEnabled);
+				pComboBox->setEditable(!bFix);
 				return pComboBox;
 			} else if (r.param.type == LSCP_TYPE_INT && bEnabled) {
 				QSpinBox *pSpinBox = new QSpinBox(pParent);
@@ -1207,9 +1204,11 @@ QWidget* DeviceParamDelegate::createEditor ( QWidget *pParent,
 						r.param.range_max.toInt() : (1 << 24) // or better a higher default max value ?
 				);
 				pSpinBox->setValue(val.toInt());
+				pSpinBox->setReadOnly(bFix);
 				return pSpinBox;
 			} else if (bEnabled) {
 				QLineEdit *pLineEdit = new QLineEdit(val, pParent);
+				pLineEdit->setReadOnly(bFix);
 				return pLineEdit;
 			} else {
 				QLabel *pLabel = new QLabel(val, pParent);
@@ -1219,7 +1218,7 @@ QWidget* DeviceParamDelegate::createEditor ( QWidget *pParent,
 		case 2:
 			return new QLabel(r.param.description, pParent);
 		default:
-			return NULL;
+			return nullptr;
 	}
 }
 
