@@ -1,7 +1,7 @@
 // qsamplerDevice.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -58,14 +58,12 @@ class DeviceParam
 public:
 
 	// Constructor.
-	DeviceParam(lscp_param_info_t *pParamInfo = NULL,
-		const char *pszValue = NULL);
-	// Default destructor.
-	~DeviceParam();
+	DeviceParam(lscp_param_info_t *pParamInfo = nullptr,
+		const char *pszValue = nullptr);
 
 	// Initializer.
 	void setParam(lscp_param_info_t *pParamInfo,
-		const char *pszValue = NULL);
+		const char *pszValue = nullptr);
 
 	// Info structure field members.
 	lscp_type_t	type;
@@ -265,7 +263,7 @@ class AbstractDeviceParamModel : public QAbstractTableModel
 
 public:
 
-	AbstractDeviceParamModel(QObject *pParent = NULL);
+	AbstractDeviceParamModel(QObject *pParent = nullptr);
 
 	// Overridden methods from subclass(es)
 	int rowCount(const QModelIndex& parent = QModelIndex()) const;
@@ -295,7 +293,7 @@ class DeviceParamModel : public AbstractDeviceParamModel
 
 public:
 
-	DeviceParamModel(QObject *pParent = NULL);
+	DeviceParamModel(QObject *pParent = nullptr);
 
 	// Overridden methods from subclass(es)
 	QVariant data(const QModelIndex &index, int role) const;
@@ -353,7 +351,7 @@ class DeviceParamDelegate : public QItemDelegate
 
 public:
 
-	DeviceParamDelegate(QObject *pParent = NULL);
+	DeviceParamDelegate(QObject *pParent = nullptr);
 
 	QWidget* createEditor(QWidget *pParent,
 		const QStyleOptionViewItem& option, const QModelIndex& index) const;
