@@ -1,7 +1,7 @@
 // qsamplerDeviceForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -47,8 +47,8 @@ DeviceForm::DeviceForm ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_iDirtyCount = 0;
 	m_bNewDevice  = false;
 	m_deviceType  = Device::None;
-	m_pAudioItems = NULL;
-	m_pMidiItems  = NULL;
+	m_pAudioItems = nullptr;
+	m_pMidiItems  = nullptr;
 	// No exclusive mode as default.
 	m_deviceTypeMode = Device::None;
 
@@ -62,7 +62,7 @@ DeviceForm::DeviceForm ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.DeviceParamTable->setModel(&m_deviceParamModel);
 	m_ui.DeviceParamTable->setItemDelegate(&m_deviceParamDelegate);
-#if QT_VERSION >= 0x050000
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 	m_ui.DeviceParamTable->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
 #else
 	m_ui.DeviceParamTable->horizontalHeader()->setResizeMode(2, QHeaderView::Stretch);
@@ -71,7 +71,7 @@ DeviceForm::DeviceForm ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.DevicePortParamTable->setModel(&m_devicePortParamModel);
 	m_ui.DevicePortParamTable->setItemDelegate(&m_devicePortParamDelegate);
-#if QT_VERSION >= 0x050000
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 	m_ui.DevicePortParamTable->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
 #else
 	m_ui.DevicePortParamTable->horizontalHeader()->setResizeMode(2, QHeaderView::Stretch);
@@ -192,7 +192,7 @@ void DeviceForm::setDevice ( Device *pDevice )
 		deviceType = pDevice->deviceType();
 
 	// Get the device view root item...
-	DeviceItem *pRootItem = NULL;
+	DeviceItem *pRootItem = nullptr;
 	switch (deviceType) {
 	case Device::Audio:
 		pRootItem = m_pAudioItems;
@@ -205,11 +205,11 @@ void DeviceForm::setDevice ( Device *pDevice )
 	}
 
 	// Is the root present?
-	if (pRootItem == NULL)
+	if (pRootItem == nullptr)
 		return;
 
 	// So there's no device huh?
-	if (pDevice == NULL) {
+	if (pDevice == nullptr) {
 		m_ui.DeviceListView->setCurrentItem(pRootItem);
 		return;
 	}
@@ -233,18 +233,18 @@ void DeviceForm::setDevice ( Device *pDevice )
 void DeviceForm::createDevice (void)
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
 
 	QTreeWidgetItem *pItem = m_ui.DeviceListView->currentItem();
-	if (pItem == NULL || pItem->type() != QSAMPLER_DEVICE_ITEM)
+	if (pItem == nullptr || pItem->type() != QSAMPLER_DEVICE_ITEM)
 		return;
 
 	// About a brand new device instance...
 	Device device(((DeviceItem *) pItem)->device());
 	if (device.createDevice()) {
 		// Now it depends on the device type...
-		DeviceItem *pRootItem = NULL;
+		DeviceItem *pRootItem = nullptr;
 		switch (device.deviceType()) {
 		case Device::Audio:
 			pRootItem = m_pAudioItems;
@@ -271,11 +271,11 @@ void DeviceForm::createDevice (void)
 void DeviceForm::deleteDevice (void)
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
 
 	QTreeWidgetItem* pItem = m_ui.DeviceListView->currentItem();
-	if (pItem == NULL || pItem->type() != QSAMPLER_DEVICE_ITEM)
+	if (pItem == nullptr || pItem->type() != QSAMPLER_DEVICE_ITEM)
 		return;
 
 	Device& device = ((DeviceItem *) pItem)->device();
@@ -283,7 +283,7 @@ void DeviceForm::deleteDevice (void)
 	// Prompt user if this is for real...
 	Options *pOptions = pMainForm->options();
 	if (pOptions && pOptions->bConfirmRemove) {
-		const QString& sTitle = QSAMPLER_TITLE ": " + tr("Warning");
+		const QString& sTitle = tr("Warning");
 		const QString& sText = tr(
 			"About to delete device:\n\n"
 			"%1\n\n"
@@ -325,7 +325,7 @@ void DeviceForm::deleteDevice (void)
 void DeviceForm::refreshDevices (void)
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
 
 	// Avoid nested changes.
@@ -334,8 +334,8 @@ void DeviceForm::refreshDevices (void)
 	//
 	// (Re)Load complete device configuration data ...
 	//
-	m_pAudioItems = NULL;
-	m_pMidiItems = NULL;
+	m_pAudioItems = nullptr;
+	m_pMidiItems = nullptr;
 	m_ui.DeviceListView->clear();
 	if (pMainForm->client()) {
 		int *piDeviceIDs;
@@ -390,7 +390,7 @@ void DeviceForm::selectDriver ( const QString& sDriverName )
 	//
 
 	QTreeWidgetItem* pItem = m_ui.DeviceListView->currentItem();
-	if (pItem == NULL || pItem->type() != QSAMPLER_DEVICE_ITEM)
+	if (pItem == nullptr || pItem->type() != QSAMPLER_DEVICE_ITEM)
 		return;
 
 	Device& device = ((DeviceItem *) pItem)->device();
@@ -411,7 +411,7 @@ void DeviceForm::selectDriver ( const QString& sDriverName )
 void DeviceForm::selectDevice ()
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
 
 	if (m_iDirtySetup > 0)
@@ -422,9 +422,9 @@ void DeviceForm::selectDevice ()
 	//
 
 	QTreeWidgetItem* pItem = m_ui.DeviceListView->currentItem();
-	if (pItem == NULL || pItem->type() != QSAMPLER_DEVICE_ITEM) {
+	if (pItem == nullptr || pItem->type() != QSAMPLER_DEVICE_ITEM) {
 		m_deviceType = Device::None;
-		m_ui.DeviceNameTextLabel->setText(QString::null);
+		m_ui.DeviceNameTextLabel->setText(QString());
 		m_deviceParamModel.clear();
 		m_ui.DevicePortComboBox->clear();
 		m_devicePortParamModel.clear();
@@ -520,11 +520,11 @@ void DeviceForm::selectDevicePort ( int iPort )
 	//
 
 	QTreeWidgetItem* pItem = m_ui.DeviceListView->currentItem();
-	if (pItem == NULL || pItem->type() != QSAMPLER_DEVICE_ITEM)
+	if (pItem == nullptr || pItem->type() != QSAMPLER_DEVICE_ITEM)
 		return;
 
 	Device& device = ((DeviceItem *) pItem)->device();
-	DevicePort *pPort = NULL;
+	DevicePort *pPort = nullptr;
 	if (iPort >= 0 && iPort < device.ports().count())
 		pPort = device.ports().at(iPort);
 	if (pPort) {
@@ -551,7 +551,7 @@ void DeviceForm::changeDeviceParam ( int iRow, int iCol )
 
 /* we do that in the model class now ...
 	QTreeWidgetItem* pItem = m_ui.DeviceListView->currentItem();
-	if (pItem == NULL || pItem->type() != QSAMPLER_DEVICE_ITEM)
+	if (pItem == nullptr || pItem->type() != QSAMPLER_DEVICE_ITEM)
 		return;
 
 	Device& device = ((DeviceItem *) pItem)->device();
@@ -590,16 +590,16 @@ void DeviceForm::changeDevicePortParam ( int iRow, int iCol )
 
 /* we do that in the model class now ...
 	QTreeWidgetItem* pItem = m_ui.DeviceListView->currentItem();
-	if (pItem == NULL || pItem->type() != QSAMPLER_DEVICE_ITEM)
+	if (pItem == nullptr || pItem->type() != QSAMPLER_DEVICE_ITEM)
 		return;
 
 	Device& device = ((DeviceItem *) pItem)->device();
 
 	int iPort = m_ui.DevicePortComboBox->currentIndex();
-	DevicePort *pPort = NULL;
+	DevicePort *pPort = nullptr;
 	if (iPort >= 0 && iPort < device.ports().count())
 		pPort = device.ports().at(iPort);
-	if (pPort == NULL)
+	if (pPort == nullptr)
 		return;
 
 	// Table 1st column has the parameter name;
@@ -626,19 +626,19 @@ void DeviceForm::changeDevicePortParam ( int iRow, int iCol )
 void DeviceForm::deviceListViewContextMenu ( const QPoint& pos )
 {
 	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm == NULL)
+	if (pMainForm == nullptr)
 		return;
 
 	QTreeWidgetItem* pItem = m_ui.DeviceListView->itemAt(pos);
-	if (pItem == NULL)
+	if (pItem == nullptr)
 		return;
 
 	// Build the device context menu...
 	QMenu menu(this);
 	QAction *pAction;
 
-	bool bClient = (pMainForm->client() != NULL);
-	bool bEnabled = (pItem != NULL);
+	bool bClient = (pMainForm->client() != nullptr);
+	bool bEnabled = (pItem != nullptr);
 	pAction = menu.addAction(
 		QIcon(":/images/deviceCreate.png"),
 		tr("&Create device"), this, SLOT(createDevice()));
@@ -662,8 +662,8 @@ void DeviceForm::stabilizeForm (void)
 {
 	MainForm* pMainForm = MainForm::getInstance();
 	QTreeWidgetItem* pItem = m_ui.DeviceListView->currentItem();
-	bool bClient = (pMainForm && pMainForm->client() != NULL);
-	bool bEnabled = (pItem != NULL);
+	bool bClient = (pMainForm && pMainForm->client() != nullptr);
+	bool bEnabled = (pItem != nullptr);
 	m_ui.DeviceNameTextLabel->setEnabled(bEnabled && !m_bNewDevice);
 	m_ui.DriverNameTextLabel->setEnabled(bEnabled &&  m_bNewDevice);
 	m_ui.DriverNameComboBox->setEnabled(bEnabled && m_bNewDevice);
