@@ -1,7 +1,7 @@
 // qsamplerOptions.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007,2008,2015 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -27,6 +27,9 @@
 #include <QTextStream>
 #include <QComboBox>
 
+#include <QApplication>
+#include <QDesktopWidget>
+
 #include <lscp/client.h>
 
 #ifdef CONFIG_LIBGIG
@@ -114,6 +117,12 @@ void Options::loadOptions (void)
 #endif
 	m_settings.endGroup();
 
+	// Load custom options...
+	m_settings.beginGroup("/Custom");
+	sCustomColorTheme = m_settings.value("/ColorTheme").toString();
+	sCustomStyleTheme = m_settings.value("/StyleTheme").toString();
+	m_settings.endGroup();
+
 	// And go into view options group.
 	m_settings.beginGroup("/View");
 	bMenubar     = m_settings.value("/Menubar", true).toBool();
@@ -206,6 +215,12 @@ void Options::saveOptions (void)
 	m_settings.setValue("/InstrumentNames", bInstrumentNames);
 	m_settings.endGroup();
 
+	// Save custom options...
+	m_settings.beginGroup("/Custom");
+	m_settings.setValue("/ColorTheme", sCustomColorTheme);
+	m_settings.setValue("/StyleTheme", sCustomStyleTheme);
+	m_settings.endGroup();
+
 	// View options group.
 	m_settings.beginGroup("/View");
 	m_settings.setValue("/Menubar", bMenubar);
@@ -286,20 +301,20 @@ bool Options::parse_args ( const QStringList& args )
 {
 	QTextStream out(stderr);
 	const QString sEol = "\n\n";
-	int iCmdArgs = 0;
 	const int argc = args.count();
+	int iCmdArgs = 0;
 
-	for (int i = 1; i < argc; i++) {
+	for (int i = 1; i < argc; ++i) {
 
 		if (iCmdArgs > 0) {
 			sSessionFile += " ";
 			sSessionFile += args.at(i);
-			iCmdArgs++;
+			++iCmdArgs;
 			continue;
 		}
 
 		QString sArg = args.at(i);
-		QString sVal = QString::null;
+		QString sVal;
 		const int iEqual = sArg.indexOf("=");
 		if (iEqual >= 0) {
 			sVal = sArg.right(sArg.length() - iEqual - 1);
@@ -321,7 +336,7 @@ bool Options::parse_args ( const QStringList& args )
 			}
 			sServerHost = sVal;
 			if (iEqual < 0)
-				i++;
+				++i;
 		}
 		else if (sArg == "-p" || sArg == "--port") {
 			if (sVal.isNull()) {
@@ -330,7 +345,7 @@ bool Options::parse_args ( const QStringList& args )
 			}
 			iServerPort = sVal.toInt();
 			if (iEqual < 0)
-				i++;
+				++i;
 		}
 		else if (sArg == "-?" || sArg == "--help") {
 			print_usage(args.at(0));
@@ -356,7 +371,7 @@ bool Options::parse_args ( const QStringList& args )
 			// If we don't have one by now,
 			// this will be the startup sesion file...
 			sSessionFile += sArg;
-			iCmdArgs++;
+			++iCmdArgs;
 		}
 	}
 
@@ -374,11 +389,23 @@ void Options::loadWidgetGeometry ( QWidget *pWidget, bool bVisible )
 	if (pWidget) {
 	//	if (bVisible) pWidget->show(); -- force initial exposure?
 		m_settings.beginGroup("/Geometry/" + pWidget->objectName());
-	#if QT_VERSION >= 0x050000
+	#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 		const QByteArray& geometry
 			= m_settings.value("/geometry").toByteArray();
-		if (!geometry.isEmpty())
+		if (geometry.isEmpty()) {
+			QWidget *pParent = pWidget->parentWidget();
+			if (pParent)
+				pParent = pParent->window();
+			if (pParent == nullptr)
+				pParent = QApplication::desktop();
+			if (pParent) {
+				QRect wrect(pWidget->geometry());
+				wrect.moveCenter(pParent->geometry().center());
+				pWidget->move(wrect.topLeft());
+			}
+		} else {
 			pWidget->restoreGeometry(geometry);
+		}
 	#else//--LOAD_OLD_GEOMETRY
 		QPoint wpos;
 		QSize  wsize;
@@ -411,7 +438,7 @@ void Options::saveWidgetGeometry ( QWidget *pWidget, bool bVisible )
 	// only save the form geometry while its up and visible)
 	if (pWidget) {
 		m_settings.beginGroup("/Geometry/" + pWidget->objectName());
-	#if QT_VERSION >= 0x050000
+	#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 		m_settings.setValue("/geometry", pWidget->saveGeometry());
 	#else//--SAVE_OLD_GEOMETRY
 		const QPoint& wpos  = pWidget->pos();
