@@ -1,7 +1,7 @@
 // qsamplerOptions.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007,2008,2015 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -91,6 +91,9 @@ public:
 	bool    bInstrumentNames;
 	int     iBaseFontSize;
 
+	QString sCustomColorTheme;
+	QString sCustomStyleTheme;
+
 	// View options...
 	bool    bMenubar;
 	bool    bToolbar;
