@@ -1,7 +1,7 @@
 // qsamplerOptionsForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -25,10 +25,18 @@
 #include "qsamplerAbout.h"
 #include "qsamplerOptions.h"
 
+#include "qsamplerPaletteForm.h"
+
 #include <QMessageBox>
 #include <QFontDialog>
 #include <QFileDialog>
 
+#include <QStyleFactory>
+
+
+// Default (empty/blank) name.
+static const char *g_pszDefName = QT_TRANSLATE_NOOP("qsamplerOptionsForm", "(default)");
+
 
 namespace QSampler {
 
@@ -42,7 +50,7 @@ OptionsForm::OptionsForm ( QWidget* pParent )
 	m_ui.setupUi(this);
 
 	// No settings descriptor initially (the caller will set it).
-	m_pOptions = NULL;
+	m_pOptions = nullptr;
 
 	// Initialize dirty control state.
 	m_iDirtySetup = 0;
@@ -133,6 +141,15 @@ OptionsForm::OptionsForm ( QWidget* pParent )
 	QObject::connect(m_ui.InstrumentNamesCheckBox,
 		SIGNAL(stateChanged(int)),
 		SLOT(optionsChanged()));
+	QObject::connect(m_ui.CustomColorThemeComboBox,
+		SIGNAL(activated(int)),
+		SLOT(optionsChanged()));
+	QObject::connect(m_ui.CustomColorThemeToolButton,
+		SIGNAL(clicked()),
+		SLOT(editCustomColorThemes()));
+	QObject::connect(m_ui.CustomStyleThemeComboBox,
+		SIGNAL(activated(int)),
+		SLOT(optionsChanged()));
 	QObject::connect(m_ui.BaseFontSizeComboBox,
 		SIGNAL(editTextChanged(const QString&)),
 		SLOT(optionsChanged()));
@@ -281,6 +298,10 @@ void OptionsForm::setup ( Options *pOptions )
 	);
 #endif // CONFIG_MAX_VOICES
 
+	// Custom display options...
+	resetCustomColorThemes(m_pOptions->sCustomColorTheme);
+	resetCustomStyleThemes(m_pOptions->sCustomStyleTheme);
+
 	// Done.
 	m_iDirtySetup--;
 	stabilizeForm();
@@ -323,6 +344,45 @@ void OptionsForm::accept (void)
 		m_pOptions->bInstrumentNames = m_ui.InstrumentNamesCheckBox->isChecked();
 		m_pOptions->iMaxRecentFiles  = m_ui.MaxRecentFilesSpinBox->value();
 		m_pOptions->iBaseFontSize  = m_ui.BaseFontSizeComboBox->currentText().toInt();
+		// Custom color/style theme options...
+		const QString sOldCustomStyleTheme = m_pOptions->sCustomStyleTheme;
+		if (m_ui.CustomStyleThemeComboBox->currentIndex() > 0)
+			m_pOptions->sCustomStyleTheme = m_ui.CustomStyleThemeComboBox->currentText();
+		else
+			m_pOptions->sCustomStyleTheme.clear();
+		const QString sOldCustomColorTheme = m_pOptions->sCustomColorTheme;
+		if (m_ui.CustomColorThemeComboBox->currentIndex() > 0)
+			m_pOptions->sCustomColorTheme = m_ui.CustomColorThemeComboBox->currentText();
+		else
+			m_pOptions->sCustomColorTheme.clear();
+		// Check whether restart is needed or whether
+		// custom options maybe set up immediately...
+		int iNeedRestart = 0;
+		if (m_pOptions->sCustomStyleTheme != sOldCustomStyleTheme) {
+			if (m_pOptions->sCustomStyleTheme.isEmpty()) {
+				++iNeedRestart;
+			} else {
+				QApplication::setStyle(
+					QStyleFactory::create(m_pOptions->sCustomStyleTheme));
+			}
+		}
+		if (m_pOptions->sCustomColorTheme != sOldCustomColorTheme) {
+			if (m_pOptions->sCustomColorTheme.isEmpty()) {
+				++iNeedRestart;
+			} else {
+				QPalette pal;
+				if (PaletteForm::namedPalette(
+						&m_pOptions->settings(), m_pOptions->sCustomColorTheme, pal))
+					QApplication::setPalette(pal);
+			}
+		}
+		// Show restart message if needed...
+		if (iNeedRestart > 0) {
+			QMessageBox::information(this,
+				tr("Information"),
+				tr("Some settings may be only effective\n"
+				"next time you start this application."));
+		}
 		// Reset dirty flag.
 		m_iDirtyCount = 0;
 	}
@@ -356,7 +416,7 @@ void OptionsForm::reject (void)
 	// Check if there's any pending changes...
 	if (m_iDirtyCount > 0) {
 		switch (QMessageBox::warning(this,
-			QSAMPLER_TITLE ": " + tr("Warning"),
+			tr("Warning"),
 			tr("Some settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
 			QMessageBox::Apply |
@@ -492,6 +552,77 @@ void OptionsForm::maxStreamsChanged(int /*iMaxStreams*/)
 	optionsChanged();
 }
 
+
+// Custom color palette theme manager.
+void OptionsForm::editCustomColorThemes (void)
+{
+	PaletteForm form(this);
+	form.setSettings(&m_pOptions->settings());
+
+	QString sCustomColorTheme;
+	int iDirtyCustomColorTheme = 0;
+
+	const int iCustomColorTheme
+		= m_ui.CustomColorThemeComboBox->currentIndex();
+	if (iCustomColorTheme > 0) {
+		sCustomColorTheme = m_ui.CustomColorThemeComboBox->itemText(
+			iCustomColorTheme);
+		form.setPaletteName(sCustomColorTheme);
+	}
+
+	if (form.exec() == QDialog::Accepted) {
+		sCustomColorTheme = form.paletteName();
+		++iDirtyCustomColorTheme;
+	}
+
+	if (iDirtyCustomColorTheme > 0 || form.isDirty()) {
+		resetCustomColorThemes(sCustomColorTheme);
+		optionsChanged();
+	}
+}
+
+
+// Custom color palette themes settler.
+void OptionsForm::resetCustomColorThemes (
+	const QString& sCustomColorTheme )
+{
+	m_ui.CustomColorThemeComboBox->clear();
+	m_ui.CustomColorThemeComboBox->addItem(
+		tr(g_pszDefName));
+	m_ui.CustomColorThemeComboBox->addItems(
+		PaletteForm::namedPaletteList(&m_pOptions->settings()));
+
+	int iCustomColorTheme = 0;
+	if (!sCustomColorTheme.isEmpty()) {
+		iCustomColorTheme = m_ui.CustomColorThemeComboBox->findText(
+			sCustomColorTheme);
+		if (iCustomColorTheme < 0)
+			iCustomColorTheme = 0;
+	}
+	m_ui.CustomColorThemeComboBox->setCurrentIndex(iCustomColorTheme);
+}
+
+
+// Custom widget style themes settler.
+void OptionsForm::resetCustomStyleThemes (
+	const QString& sCustomStyleTheme )
+{
+	m_ui.CustomStyleThemeComboBox->clear();
+	m_ui.CustomStyleThemeComboBox->addItem(
+		tr(g_pszDefName));
+	m_ui.CustomStyleThemeComboBox->addItems(QStyleFactory::keys());
+
+	int iCustomStyleTheme = 0;
+	if (!sCustomStyleTheme.isEmpty()) {
+		iCustomStyleTheme = m_ui.CustomStyleThemeComboBox->findText(
+			sCustomStyleTheme);
+		if (iCustomStyleTheme < 0)
+			iCustomStyleTheme = 0;
+	}
+	m_ui.CustomStyleThemeComboBox->setCurrentIndex(iCustomStyleTheme);
+}
+
+
 } // namespace QSampler
 
 // end of qsamplerOptionsForm.cpp
